Attribute VB_Name = "modZip"
Option Explicit

'----- zLib API
Private Type tm_struct
    tm_seconds As Long
    tm_minutes As Long
    tm_hours As Long
    tm_days As Long
    tm_months As Long
    tm_years As Long
End Type

Private Type zip_fileinfo
    tmz As tm_struct
    dosDate As Long
    internal_fa As Long
    external_fa As Long
End Type

Private Declare Function zipOpen Lib "zlib.dll" (ByVal pathname As _
    String, ByVal append As Long) As Long
Private Declare Function zipOpenNewFileInZip Lib "zlib.dll" (ByVal _
    file As Long, ByVal filename As String, zipfi As zip_fileinfo, _
    extrafield_local As Any, ByVal size_extrafield_local As Long, _
    extrafield_global As Any, ByVal size_extrafield_global As Long, _
    ByVal comment As String, ByVal method As Long, ByVal level As _
    Long) As Long
Private Declare Function zipWriteInFileInZip Lib "zlib.dll" (ByVal _
    file As Long, buf As Any, ByVal nLen As Long) As Long
Private Declare Function zipCloseFileInZip Lib "zlib.dll" (ByVal _
    file As Long) As Long
Private Declare Function zipClose Lib "zlib.dll" (ByVal file As _
    Long, ByVal comment As String) As Long

Private Const Z_DEFLATED = 8
Private Const Z_BEST_COMPRESSION = 9

Private Type unz_file_info
    version As Long
    version_needed As Long
    flag As Long
    compression_method As Long
    dosDate As Long
    crc As Long
    compressed_size As Long
    uncompressed_size As Long
    size_filename As Long
    size_file_extra As Long
    size_file_comment As Long
    disk_num_start As Long
    internal_fa As Long
    external_fa As Long
    tmu_date As tm_struct
End Type

Private Declare Function unzOpen Lib "zlib.dll" (ByVal path As _
    String) As Long
Private Declare Function unzClose Lib "zlib.dll" (ByVal file As _
    Long) As Long
Private Declare Function unzGoToFirstFile Lib "zlib.dll" (ByVal _
    file As Long) As Long
Private Declare Function unzGoToNextFile Lib "zlib.dll" (ByVal _
    file As Long) As Long
Private Declare Function unzGetCurrentFileInfo Lib "zlib.dll" _
    (ByVal file As Long, pfile_info As unz_file_info, ByVal _
    szFileName As String, ByVal fileNameBufferSize As Long, _
    extraField As Any, ByVal extraFieldBufferSize As Long, ByVal _
    szComment As String, ByVal commentBufferSize As Long) As Long
Private Declare Function unzOpenCurrentFile Lib "zlib.dll" (ByVal _
    file As Long) As Long
Private Declare Function unzCloseCurrentFile Lib "zlib.dll" (ByVal _
    file As Long) As Long
Private Declare Function unzReadCurrentFile Lib "zlib.dll" (ByVal _
    file As Long, lpvoid As Any, ByVal nLen As Long) As Long

Const UNZ_OK = 0

'--- Windows API
' These are used mostly for reading and writing files, much quicker
'  than using native VB code.

Private Type FILETIME
    dwLowDateTime As Long
    dwHighDateTime As Long
End Type

Private Declare Function FileTimeToLocalFileTime Lib "kernel32" _
    (lpFileTime As FILETIME, lpLocalFileTime As FILETIME) As Long
Private Declare Function GetFileTime Lib "kernel32" (ByVal _
    hFile As Long, lpCreationTime As Any, lpLastAccessTime As Any, _
    lpLastWriteTime As Any) As Long
Private Declare Function FileTimeToDosDateTime Lib "kernel32" _
    (lpFileTime As FILETIME, lpFatDate As Any, _
    lpFatTime As Any) As Long
Private Declare Function CreateFile Lib "kernel32" Alias _
    "CreateFileA" (ByVal lpFileName As String, ByVal _
    dwDesiredAccess As Long, ByVal dwShareMode As Long, _
    lpSecurityAttributes As Any, ByVal dwCreationDisposition _
    As Long, ByVal dwFlagsAndAttributes As Long, ByVal _
    hTemplateFile As Long) As Long
Private Declare Function CloseHandle Lib "kernel32" (ByVal _
    hObject As Long) As Long
Private Declare Function ReadFile Lib "kernel32" (ByVal _
    hFile As Long, lpBuffer As Any, ByVal nNumberOfBytesToRead _
    As Long, lpNumberOfBytesRead As Long, _
    lpOverlapped As Any) As Long
Private Declare Function WriteFile Lib "kernel32" (ByVal _
    hFile As Long, lpBuffer As Any, ByVal _
    nNumberOfBytesToWrite As Long, lpNumberOfBytesWritten As _
    Long, lpOverlapped As Any) As Long

Private Const GENERIC_READ = &H80000000
Private Const OPEN_EXISTING = 3
Private Const FILE_SHARE_READ = &H1
Private Const GENERIC_WRITE = &H40000000
Private Const CREATE_ALWAYS = 2

'Expand a zip file into a directory.
'Optionally create a list of filenames into a collection.
Public Function ExpandZipFile(sDestDir As String, sZipFileName _
    As String, Optional cFiles As Collection = Nothing) As Boolean
    On Error GoTo ErrorHandler

    If Right(sDestDir, 1) <> "\" Then
        sDestDir = sDestDir & "\"
    End If
    If Dir(sZipFileName) = "" Then
        Exit Function
    End If

    Dim bNames As Boolean
    If cFiles Is Nothing Then
        bNames = False
    Else
        bNames = True
    End If

    Dim zHandle As Long
    zHandle = unzOpen(sZipFileName)
    If zHandle = 0 Then
        Exit Function
    End If
    
    Dim nRet As Long
    nRet = unzGoToFirstFile(zHandle)
    Do While nRet = UNZ_OK
        Dim ufi As unz_file_info
        Dim sFile As String
        Dim sDir As String
        Dim vSplit As Variant
        Dim i As Long
        Dim bArray() As Byte
        Dim hFile As Long
        Dim dwTemp As Long
        
        sFile = Space(260)
        unzGetCurrentFileInfo zHandle, ufi, sFile, 260, _
            ByVal 0, 0, vbNullString, 0
        sFile = Mid(sFile, 1, InStr(1, sFile, vbNullChar) - 1)
        sFile = Replace(sFile, "/", "\")
        
        If ufi.uncompressed_size > 0 Then
            If InStr(1, sFile, "\") > 0 Then
                'See if the directory exists, if not, create it
                sDir = ""
                vSplit = Split(sFile, "\")
                For i = LBound(vSplit) To UBound(vSplit) - 1
                    If sDir <> "" Then
                        sDir = sDir & "\"
                    End If
                    sDir = sDir & vSplit(i)
                    If Dir(sDestDir & sDir, vbDirectory) = "" Then
                        MkDir sDestDir & sDir
                    End If
                Next
            End If
            
            'Unpack, and write the file to disk
            unzOpenCurrentFile zHandle
            ReDim bArray(ufi.uncompressed_size)
            unzReadCurrentFile zHandle, bArray(0), _
                ufi.uncompressed_size
            
            hFile = CreateFile(sDestDir & sFile, GENERIC_WRITE Or _
                GENERIC_READ, FILE_SHARE_READ, ByVal 0, _
                CREATE_ALWAYS, 0, 0)
            WriteFile hFile, bArray(0), ufi.uncompressed_size, _
                dwTemp, ByVal 0
            CloseHandle hFile
            
            unzCloseCurrentFile zHandle
            
        End If
        
        nRet = unzGoToNextFile(zHandle)
    Loop
    
    unzClose zHandle
    
    ExpandZipFile = True
    Exit Function
ErrorHandler:
    unzClose zHandle
End Function

'Create a zip file, taking the files mentioned in a collection,
'  in a specific directory
Public Function CreateZipFile(sRootDir As String, _
    cFiles As Collection, sZipFileName As String) As Boolean
        
    On Error GoTo ErrorHandler
    
    Do Until Right(sRootDir, 1) = "\"
        sRootDir = sRootDir & "\"
    Loop

    Dim zHandle As Long
    zHandle = zipOpen(sZipFileName, 0)
    If zHandle = 0 Then
        Exit Function
    End If

    Dim vItem As Variant
    Dim bArray() As Byte
    Dim hFile As Long
    Dim dwLen As Long
    Dim dwTemp As Long
    Dim zipfi As zip_fileinfo
    For Each vItem In cFiles
        If Dir(sRootDir & vItem) <> "" Then
            dwLen = FileLen(sRootDir & vItem)
            If dwLen <> 0 Then
                hFile = CreateFile(sRootDir & vItem, GENERIC_READ, _
                    FILE_SHARE_READ, ByVal 0, OPEN_EXISTING, 0, 0)
                ReDim bArray(dwLen)
                ReadFile hFile, bArray(0), dwLen, dwTemp, ByVal 0
                CloseHandle hFile
                
                Dim ft1 As FILETIME
                Dim ft2 As FILETIME
                GetFileTime hFile, ft1, ByVal 0, ByVal 0
                FileTimeToLocalFileTime ft1, ft2
                FileTimeToDosDateTime ft2, ByVal (VarPtr( _
                    zipfi.dosDate) + 2), ByVal VarPtr(zipfi.dosDate)
                
                zipOpenNewFileInZip zHandle, vItem, zipfi, ByVal 0, _
                    0, ByVal 0, 0, vbNullString, Z_DEFLATED, _
                    Z_BEST_COMPRESSION
                
                zipWriteInFileInZip zHandle, bArray(0), dwLen
                
                zipCloseFileInZip zHandle
                
            End If
        End If
    Next

    zipClose zHandle, vbNullString

    CreateZipFile = True
    
    Exit Function
ErrorHandler:
    zipClose zHandle, vbNullString
End Function
